#include <amxmodx>
#include <amxmisc>
#include <nvault>
#include <cstrike>

#define PLUGIN "Token System [Roulette & Blackjack]"
#define VERSION "1.0"
#define AUTHOR "Maka"

new const VAULT_FILE[] = "addons/amxmodx/data/vault/file.vault"
new const JOURNAL_FILE[] = "addons/amxmodx/data/vault/file.journal"

// Player data arrays
new g_playerTokens[33]
new g_szAuth[33][35]
new g_menuTarget[33]
new g_vault

// Game arrays
new g_PlayerBet[33]
new g_PlayerCards[33]
new g_DealerCards[33] 

// Roulette colors
enum _:RouletteColors {
    COLOR_RED,
    COLOR_BLACK,
    COLOR_GREEN
}

new const g_RouletteColors[37] = {
    COLOR_GREEN,  // 0
    COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK,  // 1-10
    COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED,  // 11-18
    COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK,  // 19-26
    COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK, COLOR_RED, COLOR_BLACK,  // 27-34
    COLOR_RED, COLOR_BLACK  // 35-36
}

public plugin_init() {
    register_plugin(PLUGIN, VERSION, AUTHOR)
    
    register_clcmd("say /tm", "CmdTokenMenu", ADMIN_CFG)
    register_clcmd("say /games", "cmd_games")
    register_clcmd("Enter_Token_Amount", "HandleTokenAmount")
    
    if (!dir_exists("addons/amxmodx/data/vault")) {
        mkdir("addons/amxmodx/data/vault")
    }
    
    g_vault = nvault_open("mcdonald_token")
    if (g_vault == INVALID_HANDLE) {
        log_amx("Creating new vault files: %s and %s", VAULT_FILE, JOURNAL_FILE)
        g_vault = nvault_open("mcdonald_token")
    }
    
    if (g_vault == INVALID_HANDLE) {
        set_fail_state("Failed to open or create vault files")
    }
}

public cmd_games(id) {
    if (!is_user_connected(id))
        return PLUGIN_HANDLED
    
    new menu_title[64]
    formatex(menu_title, charsmax(menu_title), "\r[Games] \w- Select Game \y(Tokens: %d)", g_playerTokens[id])
    new menu = menu_create(menu_title, "handle_games_menu")
    
    menu_additem(menu, "\wPlay \rBlackjack", "1")
    menu_additem(menu, "\wPlay \rRoulette", "2")
    menu_additem(menu, "\wExit", "3")
    
    menu_setprop(menu, MPROP_EXIT, MEXIT_ALL)
    menu_setprop(menu, MPROP_PERPAGE, 0)
    menu_setprop(menu, MPROP_NUMBER_COLOR, "\r")
    menu_display(id, menu, 0)
    return PLUGIN_HANDLED
}

public handle_games_menu(id, menu, item) {
    if (item == MENU_EXIT) {
        menu_destroy(menu)
        return PLUGIN_HANDLED
    }
    
    switch (item) {
        case 0: display_bet_menu(id)
        case 1: display_roulette_bet_menu(id)
    }
    
    menu_destroy(menu)
    return PLUGIN_HANDLED
}

public plugin_end() {
    nvault_close(g_vault)
}

public client_authorized(id) {
    get_user_authid(id, g_szAuth[id], charsmax(g_szAuth[]))
    g_playerTokens[id] = pb_get_token(g_szAuth[id])
}

public client_death(killer, victim, weapon, hitplace) {
    if (!is_user_connected(killer) || killer == victim)
        return
        
    pb_add_token(killer, 1)
    set_hudmessage(0, 255, 255, -1.0, 0.55, 0, 0.1, 1.0, 0.0, 0.0, -1)
    show_hudmessage(killer, "+ 1 Token")
}

public CmdTokenMenu(id, level) {
    if (!cmd_access(id, level, ADMIN_CFG, 1))
        return PLUGIN_HANDLED
    
    new menu = menu_create("\rToken Management Menu", "TokenMenuHandler")
    
    static players[32], num, i, name[32], temp[64]
    get_players(players, num)
    
    for (i = 0; i < num; i++) {
        get_user_name(players[i], name, charsmax(name))
        formatex(temp, charsmax(temp), "\w%s \y[\r%d tokens\y]", name, g_playerTokens[players[i]])
        new data[6]
        num_to_str(players[i], data, charsmax(data))
        menu_additem(menu, temp, data)
    }
    
    menu_setprop(menu, MPROP_EXIT, MEXIT_ALL)
    menu_setprop(menu, MPROP_PERPAGE, 7)
    menu_setprop(menu, MPROP_NUMBER_COLOR, "\r")
    menu_display(id, menu, 0)
    return PLUGIN_HANDLED
}

public TokenMenuHandler(id, menu, item) {
    if (item == MENU_EXIT) {
        menu_destroy(menu)
        return PLUGIN_HANDLED
    }
    
    new access, callback, data[6], name[32]
    menu_item_getinfo(menu, item, access, data, charsmax(data), name, charsmax(name), callback)
    
    g_menuTarget[id] = str_to_num(data)
    client_cmd(id, "messagemode ^"Enter_Token_Amount^"")
    
    menu_destroy(menu)
    return PLUGIN_HANDLED
}

public HandleTokenAmount(id) {
    new amount[16]
    read_args(amount, charsmax(amount))
    remove_quotes(amount)
    trim(amount)
    
    new tokens = str_to_num(amount)
    new target = g_menuTarget[id]
    
    new admin_name[32], target_name[32]
    get_user_name(id, admin_name, charsmax(admin_name))
    get_user_name(target, target_name, charsmax(target_name))
    
    if (tokens > 0) {
        pb_add_token(target, tokens)
        client_print(0, print_chat, "[Token System] Admin %s gave %d Token to %s", admin_name, tokens, target_name)
        
        // Update menus for the target player if they're in a menu
        if (is_user_connected(target)) {
            new menuId, newmenu
            player_menu_info(target, menuId, newmenu)
            
            if (menuId > 0) {
                menu_cancel(target)
                cmd_games(target)
            }
        }
    }
    
    return PLUGIN_HANDLED
}



// Blackjack menus
public display_bet_menu(id) {
    new menu_title[64]
    formatex(menu_title, charsmax(menu_title), "\r[Blackjack] \w- Select Bet \y(Tokens: %d):", g_playerTokens[id])
    new menu = menu_create(menu_title, "handle_bet_menu")
    
    menu_additem(menu, "\wBet \y50 \wTokens", "1")
    menu_additem(menu, "\wBet \y100 \wTokens", "2")
    menu_additem(menu, "\wBet \y200 \wTokens", "3")
    menu_additem(menu, "\wBet \y300 \wTokens", "4")
    menu_additem(menu, "\wExit", "5")
    
    menu_setprop(menu, MPROP_EXIT, MEXIT_ALL)
    menu_setprop(menu, MPROP_PERPAGE, 0)
    menu_setprop(menu, MPROP_NUMBER_COLOR, "\r")
    menu_display(id, menu, 0)
}

public handle_bet_menu(id, menu, item) {
    if (item == MENU_EXIT) {
        menu_destroy(menu)
        return PLUGIN_HANDLED
    }
    
    switch (item) {
        case 0: g_PlayerBet[id] = 50
        case 1: g_PlayerBet[id] = 100
        case 2: g_PlayerBet[id] = 200
        case 3: g_PlayerBet[id] = 300
        case 4: {
            menu_destroy(menu)
            return PLUGIN_HANDLED
        }
    }
    
    if (g_playerTokens[id] < g_PlayerBet[id]) {
        client_print(id, print_chat, "[Blackjack] You don't have enough tokens!")
        return PLUGIN_HANDLED
    }
    
    pb_take_token(id, g_PlayerBet[id])
    g_PlayerCards[id] = random_num(7, 11)  // Player starts with higher card
    g_DealerCards[id] = random_num(8, 11)  // Dealer starts strong
    
    client_print(id, print_chat, "[Blackjack] Your card: %d | Dealer shows: %d", g_PlayerCards[id], g_DealerCards[id])
    
    display_blackjack_game_menu(id)
    menu_destroy(menu)
    return PLUGIN_HANDLED
}

public display_blackjack_game_menu(id) {
    new menu_title[64]
    formatex(menu_title, charsmax(menu_title), "\r[Blackjack] \w- Game Options \y(Tokens: %d):", g_playerTokens[id])
    new menu = menu_create(menu_title, "handle_blackjack_game_menu")
    
    menu_additem(menu, "\wDraw \yCard", "1")
    menu_additem(menu, "\wHold \yCard", "2")
    menu_additem(menu, "\wExit", "3")
    
    menu_setprop(menu, MPROP_EXIT, MEXIT_ALL)
    menu_setprop(menu, MPROP_PERPAGE, 0)
    menu_setprop(menu, MPROP_NUMBER_COLOR, "\r")
    menu_display(id, menu, 0)
}

public handle_blackjack_game_menu(id, menu, item) {
    if (item == MENU_EXIT) {
        menu_destroy(menu)
        return PLUGIN_HANDLED
    }
    
    new player_name[32]
    get_user_name(id, player_name, charsmax(player_name))
    
    switch (item) {
        case 0: { // Draw card
            new new_card = random_num(6, 11)
            g_PlayerCards[id] += new_card
            
            client_print(id, print_chat, "[Blackjack] You drew: %d. Your total: %d | Dealer shows: %d", new_card, g_PlayerCards[id], g_DealerCards[id])
            
            if (g_PlayerCards[id] > 21) {
                client_print(0, print_chat, "[Blackjack] %s bet [%d] Token and LOST! (Bust with: %d)", player_name, g_PlayerBet[id], g_PlayerCards[id])
                return PLUGIN_HANDLED
            }
            
            if (g_PlayerCards[id] == 21) {
                client_print(0, print_chat, "[Blackjack] %s bet [%d] Token and WON with Blackjack!", player_name, g_PlayerBet[id])
                pb_add_token(id, g_PlayerBet[id] * 3)
                return PLUGIN_HANDLED
            }
            
            display_blackjack_game_menu(id)
        }
        case 1: { // Hold
            while (g_DealerCards[id] < 17) {
                new dealer_card = random_num(7, 11)
                g_DealerCards[id] += dealer_card
                client_print(id, print_chat, "[Blackjack] Dealer draws: %d. Dealer total: %d", dealer_card, g_DealerCards[id])
            }
            
            client_print(id, print_chat, "[Blackjack] Final hands - Your total: %d | Dealer total: %d", g_PlayerCards[id], g_DealerCards[id])
            
            if (g_DealerCards[id] > 21 || g_PlayerCards[id] > g_DealerCards[id]) {
                client_print(0, print_chat, "[Blackjack] %s bet [%d] Token and WON! (Player: %d | Dealer: %d)", player_name, g_PlayerBet[id], g_PlayerCards[id], g_DealerCards[id])
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Blackjack] %s bet [%d] Token and LOST! (Player: %d | Dealer: %d)", player_name, g_PlayerBet[id], g_PlayerCards[id], g_DealerCards[id])
            }
        }
    }
    
    menu_destroy(menu)
    return PLUGIN_HANDLED
}


// Roulette menus
public display_roulette_bet_menu(id) {
    new menu_title[64]
    formatex(menu_title, charsmax(menu_title), "\r[Roulette] \w- Select Bet \y(Tokens: %d):", g_playerTokens[id])
    new menu = menu_create(menu_title, "handle_roulette_bet_menu")
    
    menu_additem(menu, "\wBet \y50 \wTokens", "1")
    menu_additem(menu, "\wBet \y100 \wTokens", "2")
    menu_additem(menu, "\wBet \y200 \wTokens", "3")
    menu_additem(menu, "\wBet \y300 \wTokens", "4")
    menu_additem(menu, "\wExit", "5")
    
    menu_setprop(menu, MPROP_EXIT, MEXIT_ALL)
    menu_setprop(menu, MPROP_PERPAGE, 0)
    menu_setprop(menu, MPROP_NUMBER_COLOR, "\r")
    menu_display(id, menu, 0)
}

public handle_roulette_bet_menu(id, menu, item) {
    if (item == MENU_EXIT) {
        menu_destroy(menu)
        return PLUGIN_HANDLED
    }
    
    switch (item) {
        case 0: g_PlayerBet[id] = 50
        case 1: g_PlayerBet[id] = 100
        case 2: g_PlayerBet[id] = 200
        case 3: g_PlayerBet[id] = 300
        case 4: {
            menu_destroy(menu)
            return PLUGIN_HANDLED
        }
    }
    
    if (g_playerTokens[id] < g_PlayerBet[id]) {
        client_print(id, print_chat, "[Roulette] You don't have enough tokens!")
        return PLUGIN_HANDLED
    }
    
    pb_take_token(id, g_PlayerBet[id])
    display_roulette_game_menu(id)
    menu_destroy(menu)
    return PLUGIN_HANDLED
}

public display_roulette_game_menu(id) {
    new menu_title[64]
    formatex(menu_title, charsmax(menu_title), "\r[Roulette] \w- Place Your Bet \y(Tokens: %d)", g_playerTokens[id])
    new menu = menu_create(menu_title, "handle_roulette_game_menu")
    
    menu_additem(menu, "\wBet on \r1 to 18", "1")
    menu_additem(menu, "\wBet on \r19 to 36", "2")
    menu_additem(menu, "\wBet on \rRed Fields", "3")
    menu_additem(menu, "\wBet on \dBlack Fields", "4")
    menu_additem(menu, "\wBet on \yEven Numbers", "5")
    menu_additem(menu, "\wBet on \yOdd Numbers", "6")
    menu_additem(menu, "\wBet on \gGreen Field (0)", "7")
    menu_additem(menu, "\wExit", "8")
    
    menu_setprop(menu, MPROP_EXIT, MEXIT_ALL)
    menu_setprop(menu, MPROP_PERPAGE, 0)
    menu_setprop(menu, MPROP_NUMBER_COLOR, "\r")
    menu_display(id, menu, 0)
}

public handle_roulette_game_menu(id, menu, item) {
    if (item == MENU_EXIT) {
        menu_destroy(menu)
        return PLUGIN_HANDLED
    }
    
    new base_number = random_num(0, 36)
    new modifier = random_num(0, 10)
    new winning_number = (base_number + modifier) % 37
    
    new house_edge = random_num(1, 100)
    if (house_edge <= 55) {
        winning_number = (winning_number + random_num(1, 5)) % 37
    }
    
    new player_name[32]
    get_user_name(id, player_name, charsmax(player_name))
    
    switch (item) {
        case 0: { // 1-18
            if (winning_number >= 1 && winning_number <= 18) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on 1-18 and WON! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on 1-18 and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
        case 1: { // 19-36
            if (winning_number >= 19 && winning_number <= 36) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on 19-36 and WON! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on 19-36 and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
        case 2: { // Red
            if (g_RouletteColors[winning_number] == COLOR_RED) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on RED and WON! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on RED and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
        case 3: { // Black
            if (g_RouletteColors[winning_number] == COLOR_BLACK) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on BLACK and WON! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on BLACK and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
        case 4: { // Even
            if (winning_number > 0 && winning_number % 2 == 0) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on EVEN and WON! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on EVEN and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
        case 5: { // Odd
            if (winning_number > 0 && winning_number % 2 == 1) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on ODD and WON! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 2)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on ODD and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
        case 6: { // Green (0)
            if (winning_number == 0) {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on GREEN and WON TRIPLE! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
                pb_add_token(id, g_PlayerBet[id] * 3)
            } else {
                client_print(0, print_chat, "[Roulette] %s bet %d tokens on GREEN and LOST! (Number: %d)", player_name, g_PlayerBet[id], winning_number)
            }
        }
    }
    
    menu_destroy(menu)
    return PLUGIN_HANDLED
}

public pb_get_token(const auth[]) {
    new data[64], tokens
    nvault_get(g_vault, auth, data, charsmax(data))
    
    new token_str[16]
    strtok(data, token_str, charsmax(token_str), data, charsmax(data), '|')
    tokens = str_to_num(token_str)
    
    return tokens > 0 ? tokens : 0
}

public pb_add_token(id, amount) {
    g_playerTokens[id] += amount
    new data[64]
    formatex(data, charsmax(data), "%d|%s", g_playerTokens[id], g_szAuth[id])
    nvault_set(g_vault, g_szAuth[id], data)
    return 1
}

public pb_take_token(id, amount) {
    if (g_playerTokens[id] >= amount) {
        g_playerTokens[id] -= amount
        new data[64]
        formatex(data, charsmax(data), "%d|%s", g_playerTokens[id], g_szAuth[id])
        nvault_set(g_vault, g_szAuth[id], data)
        return 1
    }
    return 0
}

